@@ -20,6 +20,9 @@ from cryspy.C_item_loop_classes.cl_3_density_point import DensityPointL
 from cryspy.E_data_classes.cl_1_crystal import Crystal
 from cryspy.E_data_classes.cl_2_diffrn import Diffrn
 
+from cryspy.D_functions_item_loop.function_1_flip_ratio import \
+    calc_fm_perp_loc, calc_e_up_loc
+
 
 def choose_max_clambda(c_lambda: float, den: numpy.ndarray, der: numpy.ndarray,
                        rel_diff: float = 0.1) -> float:
@@ -127,11 +130,13 @@ def maximize_entropy(crystal: Crystal, l_diffrn: List[Diffrn],
             points_b=points_b, points_c=points_c,
             flag_two_channel=flag_two_channel)
 
-    l_f_nucl, l_v_2d_i, l_fr_e, l_fr_s = [], [], [], []
+    l_f_nucl, l_v_2d_i, l_fr_e, l_fr_s, l_fm_orb_perp_loc = [], [], [], [], []
     total_peaks = 0
     for diffrn in l_diffrn:
         diffrn_orient_matrix = diffrn.diffrn_orient_matrix
-        e_up = diffrn_orient_matrix.calc_e_up()
+        u_matrix = diffrn_orient_matrix.u
+        e_up = calc_e_up_loc(0., 0., 0., u_matrix)
+
         setup = diffrn.setup
         field = float(setup.field)
         h_loc = (field*e_up[0], field*e_up[1], field*e_up[2])
@@ -140,6 +145,16 @@ def maximize_entropy(crystal: Crystal, l_diffrn: List[Diffrn],
         ind_k = numpy.array(diffrn_refln.index_k, dtype=int)
         ind_l = numpy.array(diffrn_refln.index_l, dtype=int)
         total_peaks += ind_h.size
+
+        chi_m = crystal.calc_susceptibility_moment_tensor(
+            ind_h, ind_k, ind_l, flag_only_orbital=True)
+        sft_ij = chi_m[:9]
+        sftm_ij = chi_m[9:]
+
+        k_loc_i = cell.calc_k_loc(ind_h, ind_k, ind_l)
+        fm_orb_perp_loc = calc_fm_perp_loc(e_up, field, k_loc_i, sft_ij,
+                                           sftm_ij)
+
         hkl = (ind_h, ind_k, ind_l)
         fr_e = numpy.array(diffrn_refln.fr, dtype=float)
         fr_s = numpy.array(diffrn_refln.fr_sigma, dtype=float)
@@ -154,15 +169,24 @@ def maximize_entropy(crystal: Crystal, l_diffrn: List[Diffrn],
         l_v_2d_i.append((v_hkl_perp_2d_i, v_b_ferro, v_b_antiferro))
         l_fr_e.append(fr_e)
         l_fr_s.append(fr_s)
+        l_fm_orb_perp_loc.append(fm_orb_perp_loc)
 
     def temp_func(numpy_den=None):
         l_chi_sq, l_der_chi_sq = [], []
         l_der_chi_sq_f, l_der_chi_sq_a = [], []
-        for diffrn, f_nucl, v_2d_i, fr_e, fr_s in \
-                zip(l_diffrn, l_f_nucl, l_v_2d_i, l_fr_e, l_fr_s):
+        for diffrn, f_nucl, v_2d_i, fr_e, fr_s, fm_orb_perp_loc in \
+                zip(l_diffrn, l_f_nucl, l_v_2d_i, l_fr_e, l_fr_s,
+                    l_fm_orb_perp_loc):
+            diffrn_refln = diffrn.diffrn_refln
 
             f_m_perp, delta_f_m_perp, delta_f_m_perp_f, delta_f_m_perp_a = \
                 density_point.calc_fm(*v_2d_i)
+
+            # # FIXME: put condition
+            # f_m_perp = (f_m_perp[0] + fm_orb_perp_loc[0],
+            #             f_m_perp[1] + fm_orb_perp_loc[1],
+            #             f_m_perp[2] + fm_orb_perp_loc[2])
+
             fr_m, delta_fr_m = diffrn.calc_fr(cell, f_nucl, f_m_perp,
                                               delta_f_nucl=None,
                                               delta_f_m_perp=delta_f_m_perp)
@@ -173,7 +197,7 @@ def maximize_entropy(crystal: Crystal, l_diffrn: List[Diffrn],
                                           delta_f_nucl=None,
                                           delta_f_m_perp=delta_f_m_perp_a)[1]
 
-            diffrn.diffrn_refln.numpy_fr_calc = fr_m
+            diffrn_refln.numpy_fr_calc = fr_m
             chi_sq, der_chi_sq = calc_chi_sq(fr_e, fr_s, fr_m, delta_fr_m)
             der_chi_sq_f = calc_chi_sq(fr_e, fr_s, fr_m, delta_fr_m_f)[1]
             der_chi_sq_a = calc_chi_sq(fr_e, fr_s, fr_m, delta_fr_m_a)[1]
@@ -191,6 +215,8 @@ def maximize_entropy(crystal: Crystal, l_diffrn: List[Diffrn],
     delta_chi_sq_best, delta_chi_sq_f_best = None, None
     delta_chi_sq_a_best = None
 
+    mult_i = density_point.numpy_multiplicity
+
     c_lambda_min = 1e-9  # min value
     i_cycle = 0
     while i_cycle <= n_iterations:
@@ -268,20 +294,24 @@ of GoF is less than 0.001.",
                 d_info["stop"] = False
                 break
 
+        delta_chi_sq_f_mult_i = delta_chi_sq_f/mult_i
+        delta_chi_sq_a_mult_i = delta_chi_sq_a/mult_i
+
         rel_diff = 0.05
         if not(flag_two_channel):
+            delta_chi_sq_mult_i = delta_chi_sq/mult_i
             c_lambda = choose_max_clambda(c_lambda, numpy_density,
-                                          delta_chi_sq, rel_diff)
+                                          delta_chi_sq_mult_i, rel_diff)
             density_point.numpy_density = numpy_density*numpy.exp(
-                -c_lambda*delta_chi_sq)
+                -c_lambda*delta_chi_sq_mult_i)
         else:
             c_lambda = choose_max_clambda(c_lambda, numpy_density_ferro,
-                                          delta_chi_sq_f, rel_diff)
+                                          delta_chi_sq_f_mult_i, rel_diff)
 
         density_point.numpy_density_ferro = \
-            numpy_density_ferro*numpy.exp(-c_lambda*delta_chi_sq_f)
+            numpy_density_ferro*numpy.exp(-c_lambda*delta_chi_sq_f_mult_i)
         density_point.numpy_density_antiferro = \
-            numpy_density_antiferro*numpy.exp(-c_lambda*delta_chi_sq_a)
+            numpy_density_antiferro*numpy.exp(-c_lambda*delta_chi_sq_a_mult_i)
 
         density_point.renormalize_numpy_densities(
             flag_two_channel=flag_two_channel)
@@ -362,7 +392,9 @@ def refine_susceptibility(crystal: Crystal, l_diffrn: List[Diffrn],
     total_peaks = 0
     for diffrn in l_diffrn:
         diffrn_orient_matrix = diffrn.diffrn_orient_matrix
-        e_up = diffrn_orient_matrix.calc_e_up()
+        u_matrix = diffrn_orient_matrix.u
+        e_up = calc_e_up_loc(0., 0., 0., u_matrix)
+
         setup = diffrn.setup
         field = float(setup.field)
         h_loc = (field*e_up[0], field*e_up[1], field*e_up[2])
@@ -428,7 +460,7 @@ def refine_susceptibility(crystal: Crystal, l_diffrn: List[Diffrn],
         l_chi_sq = []
         l_der_chi_sq, l_der_chi_sq_f, l_der_chi_sq_a = [], [], []
 
-        # FIXME: e_up is not used. Why?
+        # FIXME: add flag for orbital magnetic moment
         for diffrn, f_nucl, fr_e, fr_s, e_up, h_loc, k_hkl, phase_3d, \
             chi_perp_ferro, chi_perp_aferro in \
             zip(l_diffrn, l_f_nucl, l_fr_e, l_fr_s, l_e_up, l_h_loc, l_k_hkl,
@@ -444,6 +476,21 @@ def refine_susceptibility(crystal: Crystal, l_diffrn: List[Diffrn],
                                      phase_3d)
             f_m_perp = calc_moment_perp(k_hkl, f_m)
 
+            # # correction on orbital moment
+            # diffrn_refln = diffrn.diffrn_refln
+            # setup = diffrn.setup
+            # field = setup.field
+            # ind_h = diffrn_refln.numpy_index_h
+            # ind_k = diffrn_refln.numpy_index_k
+            # ind_l = diffrn_refln.numpy_index_l
+            # chi_m = crystal.calc_susceptibility_moment_tensor(
+            #     ind_h, ind_k, ind_l, flag_only_orbital=True)
+            # sft_ij = chi_m[:9]
+            # sftm_ij = chi_m[9:]
+
+            # fm_orb_perp_loc = calc_fm_perp_loc(e_up, field, k_hkl, sft_ij,
+            #                                    sftm_ij)
+
             # add ferro and anti_ferro
 
             f_m_perp_sum = (
@@ -612,10 +659,12 @@ def calc_moments(fract_x, fract_y, fract_z, field_loc,
 
     moment_x, moment_y, moment_z = m_x+m_b_x, m_y+m_b_y, m_z+m_b_z
 
-    return moment_x, moment_y, moment_z 
+    return moment_x, moment_y, moment_z
 
-def calc_moments_in_unit_cell(field_loc, density_point: DensityPointL,
-                              crystal: Crystal, mem_parameters: MEMParameters):
+
+def calc_moments_in_unit_cell(density_point: DensityPointL, crystal: Crystal,
+                              mem_parameters: MEMParameters, field_loc):
+    """Calculate magnetization density in an unit cell."""
     points_a = mem_parameters.points_a
     points_b = mem_parameters.points_b
     points_c = mem_parameters.points_c
@@ -634,15 +683,16 @@ def calc_moments_in_unit_cell(field_loc, density_point: DensityPointL,
 
     return fract_x, fract_y, fract_z, moment_x, moment_y, moment_z
 
+
 def calc_section_for_mem(section: Section, density_point: DensityPointL,
-                         crystal: Crystal, mem_parameters: MEMParameters,
-                         field_loc):
+                         crystal: Crystal, mem_parameters: MEMParameters):
     """
     Calculate magnetization density of paramagnetic compound at given field.
 
     The result is written into file.
     """
-
+    field_loc = numpy.array([section.field_x, section.field_y,
+                             section.field_z], dtype=float)
     cell = crystal.cell
     atom_site = crystal.atom_site
     np_fract_x, np_fract_y, np_fract_z = section.calc_fractions(
@@ -660,23 +710,24 @@ def calc_section_for_mem(section: Section, density_point: DensityPointL,
     np_y = (numpy.array(range(section.points_y), dtype=float) -
             0.5*section.points_y) * size_y / float(section.points_y)
 
-    np_x_2d, np_y_2d = meshgrid(np_x, np_y, indexing="ij")
+    np_x_2d, np_y_2d = numpy.meshgrid(np_x, np_y, indexing="ij")
     np_x, np_y = np_x_2d.flatten(), np_y_2d.flatten()
 
     v_pos_x, v_pos_y, v_pos_z = section.calc_axes_x_y_z(cell, atom_site)
 
-    ls_out = []
-    ls_out.append(
-        f"   dist_x    dist_y        moment_x        moment_y        moment_z")
+    ls_out = ["loop_\n_section_cut_dist_x\n_section_cut_dist_y"]
+    ls_out.append("_section_cut_moment_x\n_section_cut_moment_y")
+    ls_out.append("_section_cut_moment_z")
     for _x, _y, v_m_x, v_m_y, v_m_z in zip(
             np_x, np_y, moment_x, moment_y, moment_z):
         val_x = v_pos_x[0]*v_m_x + v_pos_x[1]*v_m_y + v_pos_x[2]*v_m_z
         val_y = v_pos_y[0]*v_m_x + v_pos_y[1]*v_m_y + v_pos_y[2]*v_m_z
         val_z = v_pos_z[0]*v_m_x + v_pos_z[1]*v_m_y + v_pos_z[2]*v_m_z
-        ls_out.append(f"{_x:9.5f} {_y:9.5f} {val_x:15.10f} {val_y:15.10f} {val_z:15.10f}")
+        ls_out.append(f"{_x:9.5f} {_y:9.5f} {val_x:15.10f} {val_y:15.10f} \
+{val_z:15.10f}")
     with open(section.url_out, "w") as fid:
         fid.write("\n".join(ls_out))
-        
+
 
 def calc_densities_3d_for_mem(density_point: DensityPointL, crystal: Crystal,
                               mem_parameters: MEMParameters):
@@ -701,7 +752,7 @@ def calc_densities_3d_for_mem(density_point: DensityPointL, crystal: Crystal,
     full_space_group_symop = space_group.full_space_group_symop
 
     points_abc = (points_a, points_b, points_c)
-    
+
     density_point.calc_rbs_i(full_space_group_symop, points_a=points_a,
                              points_b=points_b, points_c=points_c)
 
@@ -732,7 +783,7 @@ def calc_densities_3d_for_mem(density_point: DensityPointL, crystal: Crystal,
 
     r_ij = (r_11, r_12, r_13, r_21, r_22, r_23, r_31, r_32, r_33)
     b_i = (b_1, b_2, b_3)
-    
+
     den_3d = transfer_to_density_3d(index_xyz, den_i, points_abc, r_ij, b_i)
     den_ferro_3d = transfer_to_density_3d(
         index_xyz, den_ferro_i, points_abc, r_ij, b_i)
